package crawler;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import outputFormat.AsosProduct;
import outputFormat.DestinationGuide;
import utility.ExcelManager;
import utility.SiteManager;

public class ASOSCrawler {
	//	output file  
	private static String outputFile = "excelfile/asos.xls";
	
	//	output image folder  
	private static String outputImageF = "excelfile/asos_img/";
	
	//	site base url
	private static String asosBase = "http://www.asos.com";
	
	// start to crawl from url
	private static String asosStart = "http://www.asos.com/Woman/Brands-At-Asos/Cat/pgehtml.aspx?cid=1340";
	
	// brand of the product
	private static String brand;
	
	// type of the product
	private static String type;
	
	// gender of the product: WOMEN MAN UNISEX
	private static String gender;
	
	// price unit of the product
	private static String price_unit = "GBP";
	
	public static void main(String[] args){
		AsosProduct ap = new AsosProduct();
		ExcelManager.prepareFile(outputFile, ap);
		gender = "WOMEN";
		getSiteContent(asosStart);
	}

	private static void getSiteContent(String url) {
		String doc = SiteManager.getDocumentFromURL(url, "Baiduspider+(+http://www.baidu.com/search/spider.htm)");
		
		// brands from the page
		//<li><a href="http://www.asos.com/Woman/Shop-By-Brand/All-Saints/Cat/pgecategory.aspx?cid=2803">ALL SAINTS</a></li>
		Pattern brandP = Pattern.compile("<li><a href=\"([^\"]*?)\">([^<]*?)</a></li>");
		Matcher brandM = brandP.matcher(doc);
		while(brandM.find()){
			String brandPage = brandM.group(1);
			brand = brandM.group(2);
			if(!brandPage.contains(asosBase))
				brandPage = asosBase + brandPage.trim();
			System.out.println("Brand: " +brand+" page: "+brandPage);
			extractData(brandPage);
		}
	}

	private static void extractData(String url) {
		String doc = SiteManager.getDocumentFromURL(url, "Baiduspider+(+http://www.baidu.com/search/spider.htm)");
		
		//types from the page
		//<div class="attributes"><a id="ctlRefine_rptAttributes_ctl00_lstAttributeItem_ctl03_hypAttributeItem" href="/Woman/Shop-By-Brand/All-Saints/Cat/pgecategory.aspx?cid=2803&amp;Rf-100=Day+Dresses&amp;sh=0">Day Dresses</a></div>
		Pattern typeP = Pattern.compile("<a id=\"(ctlRefine_rptAttributes_ctl00[^\"]*?)\" href=\"([^\"]*?)\">([^<]*?)</a>");
		Matcher typeM = typeP.matcher(doc);
		while(typeM.find()){
			String typePage = asosBase+typeM.group(2);
			type = typeM.group(3);
			System.out.println("\tType: " +type+" page: "+typePage);
			getCatalogContent(typePage);
		}
	}

	private static void getCatalogContent(String url) {
		String doc = SiteManager.getDocumentFromURL(url, "Baiduspider+(+http://www.baidu.com/search/spider.htm)");
//		System.out.println("\t\t\tCatalog search");
		//find View all on the page 
		//<a id="ctlCategory_hypViewAll" href="/Woman/Shop-By-Brand/All-Saints/Cat/pgecategory.aspx?cid=2803&amp;pge=0&amp;pgesize=35&amp;sort=-1&amp;sh=0">View All</a>
		Pattern viewAllP = Pattern.compile("<a id=\"ctlCategory_hypViewAll\" href=\"([^\"]*?)\">View All</a>");
		Matcher viewAllM = viewAllP.matcher(doc);
		if(viewAllM.find()){
			String allPage = viewAllM.group(1);
			if(!allPage.contains(asosBase))
				allPage = asosBase + allPage.trim();
			doc = SiteManager.getDocumentFromURL(allPage, "Baiduspider+(+http://www.baidu.com/search/spider.htm)");
			System.out.println("View All find go to new page: "+allPage);
		}
		
		//find the product pages
		//<div class="productimage"><a id="ctlCategory_lstWebCategoryItems_ctl00_hypProductImage" href="/All-Saints/All-Saints-Silver-Finish-Skinny-Estelle-Jean/Prod/pgeproduct.aspx?iid=212757&amp;cid=2803&amp;clr=Black&amp;sh=0"><img id="ctlCategory_lstWebCategoryItems_ctl00_image1l" src="http://www.asos.com/images/inv/K/7/35/212757/Black/image1l.jpg" alt="All Saints Silver Finish Skinny Estelle Jean" style="border-width:0px;height:230px;width:180px;" /></a></div>
		//<div class="item">
		//	<div class="productimage"><a id="ctlCategory_lstWebCategoryItems_ctl00_hypProductImage" href="/All-Saints/All-Saints-Leather-Studded-Shoulder-Bag/Prod/pgeproduct.aspx?iid=170811&amp;cid=2803&amp;clr=WashedBlack&amp;Rf-100=Bags&amp;sh=0"><img id="ctlCategory_lstWebCategoryItems_ctl00_image1l" src="http://www.asos.com/images/inv/J/11/71/170811/WashedBlack/image1l.jpg" alt="All Saints Leather Studded Shoulder Bag" style="border-width:0px;height:230px;width:180px;" /></a></div>
		Pattern productlistP = Pattern.compile("<div class=\"productimage\"><a id=\"([^\"]*?)\" href=\"([^\"]*?)\"><img id=\"([^\"]*?)\" src=\"([^\"]*?)\" alt=\"([^\"]*?)\"([^>]*?)/></a></div>");
		Matcher productlistM = productlistP.matcher(doc);
		while(productlistM.find()){
			String productPage = productlistM.group(2);
			if(!productPage.contains(asosBase))
				productPage = asosBase + productPage.trim();
			
			System.out.println("\t\tExtract content from page: "+productPage);
			getProductContent(productPage);
		}
	}

	private static void getProductContent(String url) {
		while(url.contains(" ")){
			url = url.replaceAll(" ", "%20");
		}
		
		String doc = SiteManager.getDocumentFromURL(url, "Baiduspider+(+http://www.baidu.com/search/spider.htm)");
		
		
		//find the name of the product
		//<span id="lblProductTitle" class="product_title">All Saints Silver Finish Skinny Estelle Jean</span>
		Pattern nameP = Pattern.compile("<span id=\"lblProductTitle\" class=\"product_title\">([^>]*?)</span>");
		Matcher nameM = nameP.matcher(doc);
		
		//find the description for the products
		//<span id="lblInvLongDescription" class="product_description"><table cellpadding="0" cellspacing="0" border="0"><tr valign="top"><td class="Bullet">-&nbsp;</td><td>Skinny ‘Estelle’ jean by All Saints</td></tr><tr valign="top"><td class="Bullet">-&nbsp;</td><td>Subtle matte silver glitter finish</td></tr><tr valign="top"><td class="Bullet">-&nbsp;</td><td>Zip fly and branded button fastening</td></tr></table></span>
		Pattern descP = Pattern.compile("<span id=\"lblInvLongDescription\" class=\"product_description\">" +
				"<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" +
				"(<tr valign=\"top\"><td class=\"Bullet\">-&nbsp;</td><td>([^<]*?)</td></tr>)*?" +
				"</table></span>");
		Matcher descM = descP.matcher(doc);

		
		//find the price of the product
		//<span id="lblProductPrice">$195.03</span>
		Pattern priceP = Pattern.compile("<span id=\"lblProductPrice\">&#163;([^<]*?)</span>");
		Matcher priceM = priceP.matcher(doc);
		
		

		//find and download image
		//<a id="hypMainImage" href="javascript:ShowZoomImage();" onmouseover="window.status='Click on images to enlarge them'; return true;" onmouseout="window.status=''; return true;"><img id="imgMainImage" class="content_product_images_mainimage" name="imgMainImage" src="http://images.asos.com/inv/J/11/71/170811/WashedBlack/image1xl.jpg" alt="All Saints Leather Studded Shoulder Bag" style="border-width:0px;border-width:1px;" /></a>
		Pattern imageP = Pattern.compile("<img id=\"imgMainImage\" class=\"content_product_images_mainimage\" name=\"imgMainImage\" src=\"([^\"]*?)\" alt=\"([^\"]*?)\"([^>]*?)/>");
		Matcher imageM = imageP.matcher(doc);
		
		System.out.println("\t\t\t"+AsosProduct.idCount+"-------------------------------------------------------------------------------");
		if(nameM.find() && descM.find() && priceM.find() && imageM.find()){
			String name = nameM.group(1);
			System.out.println("\t\t\tProduct name: "+name);
			
			
			String desc = "";
			String descF = descM.group(0);
			descP = Pattern.compile("<tr valign=\"top\"><td class=\"Bullet\">-&nbsp;</td><td>([^<]*?)</td></tr>");
			descM = descP.matcher(descF);
			while(descM.find()){
				desc += descM.group(1) + "<br>";
			}
			System.out.println("\t\t\tProduct description: " + desc);
			
			String price = priceM.group(1);
			System.out.println("\t\t\tProduct price: " + price +" "+ price_unit);
			
			
			String image = imageM.group(1);
			System.out.println("\t\t\tProduct image: " + image);
			try {
				image = SiteManager.saveImage(image, outputImageF, AsosProduct.idCount);
			} catch (Exception e) {
				e.printStackTrace();
			}
			AsosProduct ap = new AsosProduct(name, image, type, brand, price, price_unit, gender, desc, url);
			ExcelManager.writeToFile(ap);
		}else {
			nameM.reset();
			descM.reset();
			priceM.reset();
			imageM.reset();
			if(!nameM.find()){
				System.out.println("Name not found!");
			}
			if(!descM.find()){
				System.out.println("Description not found!");
			}
			if(!priceM.find()){
				System.out.println("price not found!");
			}
			if(!imageM.find()){
				System.out.println("image not found!");
			}
		}
		System.out.println("\t\t\t"+AsosProduct.idCount+"-------------------------------------------------------------------------------");
	}
}
